# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package DBOps;

use strict;
use warnings FATAL => 'all';
use diagnostics;

require "db.pl";




# 0: url
# 1: chan id
# 2: nick id
# 3: time = localtime()
sub add_url
{
	my ($url, $chan, $nick, $time) = @_;
	my $query;

	$url = &DB::safe_string($url);
	$chan = &DB::safe_string($chan);
	$nick = &DB::safe_string($nick);

	if (defined($time)) {
		$time = &DB::safe_string($time);
		$query = "INSERT INTO urls (url, nick, chan, time) VALUES " .
			"('$url', $nick, $chan, '$time')";
	} else {
		$query = "INSERT INTO urls (url, nick, chan) VALUES " .
			"('$url', $nick, $chan)";
	}
	
	&DB::update($query);
}


# 0: url
# 1: chan
# r: (id, count, time, nick)
sub get_url_info_by_url
{
	my ($url, $chan) = @_;
	my ($query, $result);

	$url = &DB::safe_string($url);
	$chan = &DB::safe_string($chan);

	$query = "SELECT id, count, " . &get_date_fmt("time") . ", nick " .
			"FROM urls " .
			"WHERE url = '$url' AND chan = $chan";
	$result = &DB::query($query);
	($result->ntuples == 0) && return undef;

	my ($id, $count, $time, $nick) = $result->fetchrow;
	return ($id, $count, $time, $nick);
}


# 0: url id
# r: (count, time, nick, url)
sub get_url_info_by_id
{
	my ($url_id) = @_;
	my ($query, $result);

	$url_id = &DB::safe_string($url_id);

	$query = "SELECT count, " . &get_date_fmt("time") . ", nick, url " .
			"FROM urls WHERE id = $url_id";
	$result = &DB::query($query);
	($result->ntuples == 0) && return undef;

	my ($count, $time, $nick, $url) = $result->fetchrow;
	return ($count, $time, $nick, $url);
}


# 0: field
# 1: (as = field)
# 2: (format)
# r: to_char for date
sub get_date_fmt
{
	my ($field, $as, $format) = @_;

	$field = &DB::safe_string($field);

	if (! defined($as)) {
		($as) = $field =~ /\.(.*)$/;
		$as = $field if (! defined($as));
	} else {
		$as = &DB::safe_string($as);
	}

	if (defined($format)) {
		return "TO_CHAR($field, '$format') AS $as";
	} else {
		return "TO_CHAR($field, 'Mon dd hh24:mi:ss yyyy') AS $as";
	}
}


return 1;
